@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 # -*- coding:utf-8 -*-
 # @Author: Jialiang Shi
-from packaging.version import parse
 from gerrit.projects.branches import GerritProjectBranches
 from gerrit.projects.tags import GerritProjectTags
 from gerrit.projects.commit import GerritProjectCommit
@@ -10,12 +9,12 @@ from gerrit.projects.labels import GerritProjectLabels
 from gerrit.projects.webhooks import GerritProjectWebHooks
 from gerrit.changes.change import GerritChange
 from gerrit.utils.models import BaseModel
-from gerrit.utils.exceptions import UnsupportMethod
 
 
 class GerritProject(BaseModel):
     def __init__(self, **kwargs):
-        super(GerritProject, self).__init__(**kwargs)
+        super().__init__(**kwargs)
+        self.endpoint = f"/projects/{self.id}"
 
     def get_description(self):
         """
@@ -23,10 +22,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/description" % self.id
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + "/description")
 
     def set_description(self, input_):
         """
@@ -42,16 +38,12 @@ class GerritProject(BaseModel):
             result = project.set_description(input_)
 
         :param input_: the ProjectDescriptionInput entity,
-          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#project-description-input
+          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html
+          #project-description-input
         :return:
         """
-        endpoint = "/projects/%s/description" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.put(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.put(self.endpoint + "/description",
+                               json=input_, headers=self.gerrit.default_headers)
 
     def delete_description(self):
         """
@@ -59,8 +51,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/description" % self.id
-        self.gerrit.requester.delete(self.gerrit.get_endpoint_url(endpoint))
+        self.gerrit.delete(self.endpoint + "/description")
 
     def delete(self):
         """
@@ -68,19 +59,16 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/delete-project~delete" % self.id
-        self.gerrit.requester.post(self.gerrit.get_endpoint_url(endpoint))
+        self.gerrit.post(self.endpoint + "/delete-project~delete")
 
     def get_parent(self):
         """
-        Retrieves the name of a project’s parent project. For the All-Projects root project an empty string is returned.
+        Retrieves the name of a project’s parent project. For the All-Projects root project an empty
+        string is returned.
 
         :return:
         """
-        endpoint = "/projects/%s/parent" % self.id
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + "/parent")
 
     def set_parent(self, input_):
         """
@@ -96,16 +84,12 @@ class GerritProject(BaseModel):
             result = project.set_parent(input_)
 
         :param input_: The ProjectParentInput entity,
-          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#project-parent-input
+          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html
+          #project-parent-input
         :return:
         """
-        endpoint = "/projects/%s/parent" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.put(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.put(self.endpoint + "/parent",
+                               json=input_, headers=self.gerrit.default_headers)
 
     def get_HEAD(self):
         """
@@ -113,10 +97,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/HEAD" % self.id
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + "/HEAD")
 
     def set_HEAD(self, input_):
         """
@@ -134,26 +115,19 @@ class GerritProject(BaseModel):
           https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#head-input
         :return:
         """
-        endpoint = "/projects/%s/HEAD" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.put(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.put(self.endpoint + "/HEAD",
+                               json=input_, headers=self.gerrit.default_headers)
 
     def get_config(self):
         """
         Gets some configuration information about a project.
-        Note that this config info is not simply the contents of project.config; it generally contains fields that may
+        Note that this config info is not simply the contents of project.config; it generally
+        contains fields that may
         have been inherited from parent projects.
 
         :return:
         """
-        endpoint = "/projects/%s/config" % self.id
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + "/config")
 
     def set_config(self, input_):
         """
@@ -182,13 +156,8 @@ class GerritProject(BaseModel):
           https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#config-info
         :return:
         """
-        endpoint = "/projects/%s/config" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.put(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.put(self.endpoint + "/config",
+                               json=input_, headers=self.gerrit.default_headers)
 
     def get_statistics(self):
         """
@@ -196,10 +165,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/statistics.git" % self.id
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + "/statistics.git")
 
     def run_garbage_collection(self, input_):
         """
@@ -217,13 +183,8 @@ class GerritProject(BaseModel):
           https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#gc-input
         :return:
         """
-        endpoint = "/projects/%s/gc" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.post(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.post(self.endpoint + "/gc",
+                                json=input_, headers=self.gerrit.default_headers)
 
     def ban_commits(self, input_):
         """
@@ -245,13 +206,8 @@ class GerritProject(BaseModel):
           https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#ban-input
         :return:
         """
-        endpoint = "/projects/%s/ban" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.put(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.put(self.endpoint + "/ban",
+                               json=input_, headers=self.gerrit.default_headers)
 
     def get_access_rights(self):
         """
@@ -259,10 +215,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/access" % self.id
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + "/access")
 
     def set_access_rights(self, input_):
         """
@@ -270,21 +223,18 @@ class GerritProject(BaseModel):
         https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#set-access
 
         :param input_: the ProjectAccessInput entity,
-          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#project-access-input
+          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html
+          #project-access-input
         :return:
         """
-        endpoint = "/projects/%s/access" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.post(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.post(self.endpoint + "/access",
+                                json=input_, headers=self.gerrit.default_headers)
 
     def create_change(self, input_):
         """
-        Create Change for review. This endpoint is functionally equivalent to create change in the change API,
-        but it has the project name in the URL, which is easier to route in sharded deployments.
+        Create Change for review. This endpoint is functionally equivalent to create change in the
+        change API, but it has the project name in the URL, which is easier to route in sharded
+        deployments.
         support this method since v3.3.0
 
         .. code-block:: python
@@ -302,36 +252,26 @@ class GerritProject(BaseModel):
         :param input_:
         :return:
         """
-        version = self.gerrit.version
-        if parse(version) < parse("3.3.0"):
-            raise UnsupportMethod("The server does not support this method")
-
-        endpoint = "/projects/%s/create.change" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.post(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return GerritChange.parse(result, gerrit=self.gerrit)
+        result = self.gerrit.post(self.endpoint + "/create.change",
+                                  json=input_, headers=self.gerrit.default_headers)
+        return GerritChange(json=result, gerrit=self.gerrit)
 
     def create_access_rights_change(self, input_):
         """
         Sets access rights for the project using the diff schema provided by ProjectAccessInput
-        This takes the same input as Update Access Rights, but creates a pending change for review. Like Create Change,
-        it returns a ChangeInfo entity describing the resulting change.
-        https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#create-access-change
+        This takes the same input as Update Access Rights, but creates a pending change for review.
+        Like Create Change, it returns a ChangeInfo entity describing the resulting change.
+        https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html
+        #create-access-change
 
         :param input_: the ProjectAccessInput entity,
-          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#project-access-input
+          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html
+          #project-access-input
         :return:
         """
-        endpoint = "/projects/%s/access:review" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.put(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return GerritChange.parse(result, gerrit=self.gerrit)
+        result = self.gerrit.put(self.endpoint + "/access:review",
+                                 json=input_, headers=self.gerrit.default_headers)
+        return GerritChange(json=result, gerrit=self.gerrit)
 
     def check_access(self, options):
         """
@@ -342,20 +282,18 @@ class GerritProject(BaseModel):
             * Account(account): The account for which to check access. Mandatory.
             * Permission(perm): The ref permission for which to check access.
               If not specified, read access to at least branch is checked.
-            * Ref(ref): The branch for which to check access. This must be given if perm is specified.
+            * Ref(ref): The branch for which to check access. This must be given if perm is
+            specified.
 
         :return:
         """
-        endpoint = "/projects/%s/check.access?%s" % (self.id, options)
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + f"/check.access?{options}")
 
     def index(self, input_):
         """
         Adds or updates the current project (and children, if specified) in the secondary index.
-        The indexing task is executed asynchronously in background and this command returns immediately
-        if async is specified in the input.
+        The indexing task is executed asynchronously in background and this command returns
+        immediately if async is specified in the input.
 
         .. code-block:: python
 
@@ -367,25 +305,22 @@ class GerritProject(BaseModel):
             result = project.index(input_)
 
         :param input_: the IndexProjectInput entity,
-          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#index-project-input
+          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html
+          #index-project-input
         :return:
         """
-        endpoint = "/projects/%s/index" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        self.gerrit.requester.post(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
+        self.gerrit.post(self.endpoint + "/index",
+                         json=input_, headers=self.gerrit.default_headers)
 
     def index_all_changes(self):
         """
         Adds or updates the current project (and children, if specified) in the secondary index.
-        The indexing task is executed asynchronously in background and this command returns immediately
-        if async is specified in the input.
+        The indexing task is executed asynchronously in background and this command returns
+        immediately if async is specified in the input.
 
         :return:
         """
-        endpoint = "/projects/%s/index.changes" % self.id
-        self.gerrit.requester.post(self.gerrit.get_endpoint_url(endpoint))
+        self.gerrit.post(self.endpoint + "/index.changes")
 
     def check_consistency(self, input_):
         """
@@ -405,16 +340,12 @@ class GerritProject(BaseModel):
             result = project.check_consistency(input_)
 
         :param input_: the CheckProjectInput entity,
-          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html#check-project-input
+          https://gerrit-review.googlesource.com/Documentation/rest-api-projects.html
+          #check-project-input
         :return:
         """
-        endpoint = "/projects/%s/check" % self.id
-        base_url = self.gerrit.get_endpoint_url(endpoint)
-        response = self.gerrit.requester.post(
-            base_url, json=input_, headers=self.gerrit.default_headers
-        )
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.post(self.endpoint + "/check",
+                                json=input_, headers=self.gerrit.default_headers)
 
     @property
     def branches(self):
@@ -432,10 +363,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/children/" % self.id
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return result
+        return self.gerrit.get(self.endpoint + "/children/")
 
     @property
     def tags(self):
@@ -452,10 +380,8 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        endpoint = "/projects/%s/commits/%s" % (self.id, commit)
-        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-        result = self.gerrit.decode_response(response)
-        return GerritProjectCommit.parse(result, project=self.id, gerrit=self.gerrit)
+        result = self.gerrit.get(self.endpoint + f"/commits/{commit}")
+        return GerritProjectCommit(json=result, project=self.id, gerrit=self.gerrit)
 
     @property
     def dashboards(self):
@@ -474,9 +400,7 @@ class GerritProject(BaseModel):
         """
         version = self.gerrit.version
         if parse(version) < parse("3.2.0"):
-            endpoint = "/projects/%s" % self.id
-            response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
-            result = self.gerrit.decode_response(response)
+            result = self.gerrit.get(self.endpoint)
             return result.get("labels")
         else:
             return GerritProjectLabels(project=self.id, gerrit=self.gerrit)
